; Ruby4mIRC support script for version 1.1
; 
; Written by Loren Segal, clb@soen.ca
;
; This file is not needed to use ruby4mirc.dll but 
; provides a simplified interface to access it.
;
; See USAGE.txt for information on how to use
; the commands defined here, or look at the
; Examples below.

; Convenience methods
alias rbdll { return $qt($scriptdirruby4mirc.dll) }
alias rbstr { return ' $+ $replacex($1-,',\',\,\\) $+ ' }
alias rb { if ($isid) return $rb.eval($1-) | dll $rbdll rb_eval $1- }
alias rb.async { noop $dllcall($rbdll,$iif(--* iswm $1,$right($1,-2),noop), rb_eval, $iif(--* iswm $1,$2-,$1-)) }
alias rbunload { dll -u $rbdll }

; Ruby DLL bridge methods
alias rb.send { return $dll($rbdll,rb_send,$1-) }
alias rb.eval { return $dll($rbdll,rb_eval,$1-) }
alias rb.load { return $rb.send(load, $1-) }
alias rb.require { return $rb.send(require, $1-) } 

; Ruby interpeter bridge methods
alias invoke_ruby_interpreter { rb RubyBridge.execute_method( $rbstr($1) , $2 , $3 ) | return $false }
alias has_cached_ruby { return $!invoke_ruby_interpreter($script,$scriptline,true) }
alias has_ruby { return $!invoke_ruby_interpreter($script,$scriptline,false) }

; Error callback
on *:SIGNAL:RUBY_ERROR:echo $color(info) -ae * Ruby4mIRC: Critical Error: $1- | dll -u $rbdll

; Initialization callback
on *:SIGNAL:RUBY_LOADED: {
  echo $color(info2) -ae * Loaded Ruby4mIRC version $rb($!RUBYDLL_VERSION) $+ . Edit line $scriptline of $qt($remove($script,$mircdir)) to change this message.
  if (!$1) {
    ; Ruby4mIRC is not in supported mode
    echo $color(info) -ae * Ruby is not running in supported mode. Your version of mIRC ( $+ $version $+ ) is not compatible with Ruby4mIRC.
  }
}
on *:LOAD: { echo $color(info2) -ae * Running /testrb to see if Ruby works: | testrb }

; Standard input/output handling
on *:SIGNAL:RUBY_STDOUT:if ($1 != $null) echo -a $1-
on *:SIGNAL:RUBY_STDERR_BEGIN: { linesep -a | set %__ruby_stderr $true }
on *:SIGNAL:RUBY_STDERR_END:linesep -a
on *:SIGNAL:RUBY_STDERR: { echo $color(info) -a $iif(%__ruby_stderr,ruby:) $1- | unset %__ruby_stderr }

;
; Examples
;

; One-liners

; Classic hello world
alias ruby_hello_world { rb puts "Hello world" }

; Timer-delays the Ruby way (Experimental async call)
alias ruby_sleep { rb.async sleep(1); puts 'One second later!' }

; Download a file from a website and print the first line (needs Ruby installed)
alias ruby_get_versions_file {
  rb.require 'open-uri' 
  rb puts open('http://www.mirc.co.uk/versions.txt').read.split(/\r?\n/).first
}

; Reverses an input string the Ruby way, showing how Ruby can be made to
; access input from $1-
alias ruby_reverse {
  set %data $1-
  rb mirc[:data] = mirc[:data].reverse
  echo -a %data
  unset %data
}

; Embedded Ruby

; Test method
alias testrb {
  if $($has_ruby,2) {
    mirc("linesep -a")
    puts('Testing Ruby... here is a sorted array: %s' % [3,2,1].sort.inspect)
    if mirc.sorttok("1 6 2", 32) == "1 2 6" 
    	puts "Ruby works great!"
    end
    mirc("linesep -a")
  }
}

; Note that mIRC's parser will incorrectly indent your code if you dont leave spaces
; around your { } 's. 
alias ruby_phonebook {
  if $($has_ruby,2) {
    phonebook = {
      'Sally Smart' => '555-9999', 
      'John Doe' => '555-1212', 
      'J. Random Hacker' => '553-1337'
    }
    phonebook.each { |name,number| puts "#{name} can be reached at #{number}" }
  }
}

; Loop over channels
alias ruby_channels {
  %chans = ""
  if $($has_ruby,2) {
    mirc.chan(0).times { |i| 
      mirc[:chans] += " " + mirc.chan(i+1)
    }
  }
  echo -a %chans
  unset %chans
}

; If you absolutely must use an odd number of {}'s, you can use the following hack to 
; confuse mIRC into readjusting the bracket count:
alias ruby_bracket_hack {
  if $($has_ruby,2) { 
    # { <-- you have to open the bracket BEFORE your code, not after!
    puts "herbie } the best bot on earth." # Now we can use that bracket freely

    # We can also just escape it if it's a string
    puts "Really, \} is a cool character."

    # Remember that blocks that take |vars| have to be spaced from the bracket
    # Unless you want to use the comment hack
    [1,2,3].collect { |x| x + 3 } 

    # We can also use this to have mIRC properly indent our if statements
    # instead of forcing a tab character with ctrl-i
    if "bob".reverse == "bob" # {
      puts "Bob is a palindrome!"
    end  # }
  }    
}

; Here is an example of an mIRC getting event specific identifiers
on *:TEXT:*hello*:#ruby-ruby-moo-moo: {
  if $($has_ruby,2) {
    puts(mirc.nick + " just said hi on " + mirc.chan)
  }
}

; Simple IRB implementation
alias irb { %__irb = 0 | window -e @irb }
on *:INPUT:@irb: {
  echo @irb $+(irb(main):,$base(%__irb,10,10,3),>) $1-
  echo @irb => $rb(( $1- ).inspect)
  inc %__irb
  halt
}